from django.test import TestCase, RequestFactory, Client
from django_aux.models import *
from .models import Person
from .views import PersonLookup
from .filters import PersonFilter
from django.contrib.auth.models import User

#------------Django-Aux TESTS------------


class TestSaveFilterMixin(TestCase):
    ''' Test Case for SaveFilterMixin '''

    def setUp(self):
        # create a request factory that all tests have access to
        self.factory = RequestFactory()
        self.client = Client()
        self.user = User.objects.create_user('billy', password = 'password')
        self.client.login(username='billy', password='password')
        

    def test_savefiltermixin_get(self):
        response = self.client.get('/person-lookup')
        self.assertIsNone(response.wsgi_request.session.get('PersonLookup_qstr'))
        response = self.client.get('/person-lookup', {'last_name__icontains': 'hyatt', 'first_name__icontains':'jordan'})
        qstr = response.wsgi_request.session.get('PersonLookup_qstr')
        self.assertEqual(qstr, 'last_name__icontains=hyatt&first_name__icontains=jordan')
        response = self.client.get('/person-lookup')
        qstr = response.wsgi_request.session.get('PersonLookup_qstr')
        self.assertEqual(qstr, 'last_name__icontains=hyatt&first_name__icontains=jordan')
        response = self.client.get('/person-lookup', {'clear_filter': 'Clear Filter'})
        qstr = response.wsgi_request.session.get('PersonLookup_qstr')
    
    def test_clear_filter(self):
        request = self.factory.get('/person-lookup', {'last_name__icontains': 'hyatt', 'first_name__icontains':'jordan'})
        request.user = self.user
        view = PersonLookup()
        view.setup(request)
        kwargs = view.get_filterset_kwargs(PersonFilter)
        self.assertNotEqual(kwargs, {})
        request = self.factory.get('/person-lookup', {'last_name__icontains': 'hyatt', 'clear_filter': 'Clear Filter'})
        request.user = self.user
        view = PersonLookup()
        view.setup(request)
        kwargs = view.get_filterset_kwargs(PersonFilter)
        self.assertEqual(kwargs, {})




from django.db import connection

def create_view_from_qs(qs, view_name, sql_permissions=None):
    """Utility function to create a DB view from a queryset.

    Args:
        qs (django.db.QuerySet): The queryset to be translated into a view
        view_name (str): View name to be created/updated
        sql_permissions (str, optional): SQL to be executed after the view is created, 
            intended for setting permissions but could be anything. 
            i.e. ALTER TABLE public.{view_name} OWNER TO postgres;
    """    
    qstr, params = qs.query.sql_with_params()
    drop_qstr = f''' DROP VIEW IF EXISTS "{view_name}" '''
    qstr = f''' CREATE OR REPLACE VIEW "{view_name}" AS {qstr} '''
    with connection.cursor() as cursor:
        cursor.execute(drop_qstr)
        cursor.execute(qstr, params)
    if sql_permissions:
        with connection.cursor() as cursor:
            cursor.execute(sql_permissions)








# Generated by Django 4.0.6 on 2022-10-08 17:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_aux_geo', '0002_subdivision'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subdivision',
            name='iso_code',
            field=models.CharField(max_length=10, unique=True),
        ),
    ]



from django_filters.views import FilterView
from django_aux.views import SaveFilterMixin
from .tables import *
from .filters import *
from .models import *
from .forms import *




class PersonLookup(SaveFilterMixin, FilterView):
    model = Person
    table_class = PersonTable
    filterset_class = PersonFilter
    template_name = "test.html"